# Makefile

VERSION = 0.5

##################################################################################
# Adjust this section to suit your system

# Note: the option -R<dir> adds <dir> to the library search path when the application starts up.
# This saves having to put all these directories in LD_LIBRARY_PATH.

# SYBASE should be set to the base of your Sybase client installation
SYBASE_INCLUDES = -I$(SYBASE)/include
SYBASE_LIBS = -R$(SYBASE)/lib -L$(SYBASE)/lib -lct -lcs -lcomn -ltcl -lcomn -lintl

# QTDIR should be set to the location of your QT installation
QT_INCLUDES = -I$(QTDIR)/include
QT_LIBS = -R$(QTDIR)/lib -L$(QTDIR)/lib -lqt
MOC = $(QTDIR)/bin/moc

# You definitely need to change these next two:
# Location of dtable.h and sybint.h - from Tim's Utility library
TJCUTIL_INCLUDES = -I/home/tim/c/tjcutil
# Location of tjcutil.so - from Tim's Utility library
TJCUTIL_LIBDIR = /home/tim/c/tjcutil
TJCUTIL_LIBS = -R$(TJCUTIL_LIBDIR) -L$(TJCUTIL_LIBDIR) -ltjcutil

# You probably won't need to change these
SYSTEM_INCLUDES =
SYSTEM_LIBS = -Bdynamic -lnsl -ldl -lm -lposix4 -lnsl -lsocket -lCrun

# End of user adjustment section
##################################################################################

ALL_INCLUDES = $(TJCUTIL_INCLUDES) $(SYSTEM_INCLUDES) $(SYBASE_INCLUDES) $(QT_INCLUDES)
ALL_LIBS = $(TJCUTIL_LIBS) $(QT_LIBS) $(SYBASE_LIBS) $(SYSTEM_LIBS)

SHELL = /bin/sh

CXX = CC
LD = CC

CXXFLAGS = -g
LDFLAGS = -g

OBJS = main.o \
	db_browser.moc.o db_browser.o\
	login_dlg.moc.o login_dlg.o\
	sproc_dlg.moc.o sproc_dlg.o\
	sqlquery_dlg.moc.o sqlquery_dlg.o
TARGET = sybesque


.SUFFIXES:
.SUFFIXES: .c .cpp .o 

.cpp.o:
	$(CXX) $(CXXFLAGS) $(ALL_INCLUDES) -c $<

# targets
all: $(TARGET)

$(TARGET): $(OBJS)
	@rm -f $@
	$(LD) $(LDFLAGS) $(OBJS) -o $@ $(ALL_LIBS)

foxtester.moc.cpp: foxtester.h
	$(MOC) $? -o $@

db_browser.moc.cpp: db_browser.h
	$(MOC) $? -o $@

login_dlg.moc.cpp: login_dlg.h
	$(MOC) $? -o $@

sproc_dlg.moc.cpp: sproc_dlg.h
	$(MOC) $? -o $@

sqlquery_dlg.moc.cpp: sqlquery_dlg.h
	$(MOC) $? -o $@

clean:
	rm -f $(TARGET) *.o *.moc.*
	rm -fr SunWS_cache

install:
	echo "There is no installation step"
	echo "Copy the sybesque executable somewhere useful"

# Dependancies
db_browser.moc.o: db_browser.moc.cpp db_browser.h sproc_dlg.h sqlquery_dlg.h
login_dlg.moc.o: login_dlg.moc.cpp login_dlg.h
sproc_dlg.moc.o: sproc_dlg.moc.cpp sproc_dlg.h
sqlquery_dlg.moc.o: sqlquery_dlg.moc.cpp sqlquery_dlg.h
login_dlg.o: login_dlg.cpp login_dlg.h 
sproc_dlg.o: sproc_dlg.cpp sproc_dlg.h 
sqlquery_dlg.o: sqlquery_dlg.cpp sqlquery_dlg.h 
