/***************************************************************************
                          db_browser.cpp  -  description
                             -------------------
    begin                : Tue Jan 23 2001 by Tim Colgate
    copyright            : (C) 2001 Tim Colgate

 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *

 ***************************************************************************/
#include <sys/types.h>	// mkdir
#include <sys/stat.h>	// mkdir
#include <stdlib.h>	// system
#include <fcntl.h>	// creat
#include <errno.h>

#include <iostream>
#include <sstream>
#include <fstream>
#include <memory>
using namespace std;

#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qclipboard.h>
#include <qapplication.h>
#include <qtextstream.h>

#include "db_browser.h"
#include "db_browserdata.cpp"

DB_browser::DB_browser(QWidget *parent, const char *name) : QDialog(parent,name,true)
{
	initDialog();

	// modes
	cmbMode->insertItem("Tables");
	cmbMode->insertItem("System tables");
	cmbMode->insertItem("Stored procedures");
	cmbMode->insertItem("Triggers");
	//cmbMode->insertItem("Views");
	cmbMode->setEditable(false);
/*	cmbMode->insertItem("Indexes");
	cmbMode->insertItem("Rules");
	cmbMode->insertItem("Types");
*/

	// Dialog editor doesn't support QTable, so add it to dialog here.
	// Note that it overlays a QMultiLineEdit. Use QTable to display tabular data
	// and QMultiLineEdit to display text (e.g. text of stored procedure)
	tblResults= new DBTable(this,"NoName");
	tblResults->setGeometry(240,10,550,630);
	tblResults->setMinimumSize(0,0);
	tblResults->hide();
//	tblResults->setReadOnly(true);

	// set up status bar
	sb = new QStatusBar(this);
	sb->setGeometry(0,405,570,25);
	lblMessage = new QLabel("", sb);
	sb->addWidget(lblMessage, 5);
	lblMode = new QLabel("", sb);
	sb->addWidget(lblMode, 1, true);
	lblColumns = new QLabel("", sb);
	sb->addWidget(lblColumns, 1, true);
	lblRows = new QLabel("0 rows", sb);
	sb->addWidget(lblRows, 1, true);

	// populate right-click popup menus
	mnuDB = new QPopupMenu(this);
	mnuDB->insertItem("Login to database", this, SLOT(LoginDB()));
	mnuDB->insertSeparator();
	mnuDB->insertItem("Dump tables' DDL to directory", this, SLOT(DumpDDLDir()));	// create one file per table
	mnuDB->insertItem("Dump tables' DDL to file", this, SLOT(DumpDDLFile()));
	mnuDB->insertItem("Dump SP text to directory", this, SLOT(DumpSPDir()));	// create one file per stored proc
	mnuDB->insertItem("Dump SP text to file", this, SLOT(DumpSPFile()));
	mnuDB->insertItem("Dump table data to directory", this, SLOT(DumpDataDir()));	// no point dumping all data to one file
	mnuDB->insertSeparator();
	mnuDB->insertItem("Dump below list to clipboard", this, SLOT(CopyList()));

	mnuTable = new QPopupMenu(this);
	mnuTable->insertItem("&Refresh", this, SLOT(RefreshResults()) /*, CTRL+Key_R, ID_APP_RATESTAMPER*/);
	mnuTable->insertSeparator();
	mnuTable->insertItem("&Query", this, SLOT(ExecQuery()));
	mnuTable->insertItem("&Update", this, SLOT(ExecUpdate()));
	mnuTable->insertItem("&Delete", this, SLOT(ExecDelete()));
	mnuTable->insertItem("&Truncate", this, SLOT(ExecTruncate()));
	mnuTable->insertItem("D&rop table", this, SLOT(ExecDropTable()));
/*	mnuTable->insertSeparator();
	mnuTable->insertItem("&bcp in", this, SLOT(bcpIn()));
	mnuTable->insertItem("&bcp out", this, SLOT(bcpOut()));
*/	mnuTable->insertSeparator();
	mnuTable->insertItem("&Execute SQL", this, SLOT(ExecSql()));

	mnuSProc = new QPopupMenu(this);
	mnuSProc->insertItem("&Execute", this, SLOT(ExecSProcParam()));
	mnuSProc->insertSeparator();
	mnuSProc->insertItem("&Execute SQL", this, SLOT(ExecSql()));
	mnuSProcParam = new QPopupMenu(this);
	mnuSProcParam->insertItem("Execute with &params", this, SLOT(ExecSProcParam()));
	mnuSProcParam->insertSeparator();
	mnuSProcParam->insertItem("&Execute SQL", this, SLOT(ExecSql()));

	mnuGrid = new QPopupMenu(this);
	mnuGrid->insertItem("Copy cell", this, SLOT(CopyCell()));
	mnuGrid->insertItem("Copy row", this, SLOT(CopyRow()));
	mnuGrid->insertItem("Copy column", this, SLOT(CopyCol()));
	mnuGrid->insertItem("Copy whole table", this, SLOT(CopyTable()));
	mnuGrid->insertItem("Copy column headers", this, SLOT(CopyColHdr()));
	mnuGrid->insertSeparator();
	mnuGrid->insertItem("Size column to fit", this, SLOT(SizeCol()));
	mnuGrid->insertItem("Size all columns to fit", this, SLOT(SizeAllCol()));
	mnuGrid->insertSeparator();
	mnuGrid->insertItem("Count column", this, SLOT(CountCol()));

	init_sql();

	connect(btnClose, SIGNAL(clicked()), this, SLOT(accept()));
	connect(cmbMode, SIGNAL(activated(const QString&)), this, SLOT(changeMode(const QString&)));
	connect(btnView, SIGNAL(clicked()), this, SLOT(changeViewMode()));
 	connect(lstServer, SIGNAL(highlighted(const QString&)), this, SLOT(openServer(const QString&)));
 	connect(lstDatabase, SIGNAL(highlighted(const QString&)), this, SLOT(openDatabase(const QString&)));
 	connect(lstDatabase, SIGNAL(rightButtonPressed(QListBoxItem*, const QPoint&)), this, SLOT(DBRightClick(QListBoxItem*, const QPoint&)));
 	connect(lstTable, SIGNAL(highlighted(const QString&)), this, SLOT(showText(const QString&)));
 	connect(lstTable, SIGNAL(rightButtonPressed(QListBoxItem*, const QPoint&)), this, SLOT(tableRightClick(QListBoxItem*, const QPoint&)));
 	connect(tblResults, SIGNAL(pressed(int, int, int, const QPoint&)), this, SLOT(gridMousePressed(int, int, int, const QPoint&)));
 	connect(tblResults, SIGNAL(released(int, int, int, const QPoint&)), this, SLOT(gridMouseReleased(int, int, int, const QPoint&)));

	load_settings();	// load username/password info from disk
}

DB_browser::~DB_browser()
{
}

void DB_browser::resizeEvent(QResizeEvent*)
{
	lstTable->resize(220, height()-260);
	lstTable->repaint();
	txtResults->resize(width()-250, height()-40);
	txtResults->repaint();
	tblResults->resize(width()-250, height()-40);
	tblResults->repaint();	// required?
	sb->setGeometry(0, height()-25, width(), 25);
}

void DB_browser::DBRightClick(QListBoxItem* item, const QPoint& pt)
{
	if (item)
		mnuDB->popup(pt);
	else
		sb->message("Invalid click point");
}

void DB_browser::tableRightClick(QListBoxItem* item, const QPoint& pt)
{
	if (item) {
		//sb->message(item->text());
		if (cmbMode->currentText() == "Tables" || cmbMode->currentText() == "System tables")
			mnuTable->popup(pt);
		else if (cmbMode->currentText() == "Stored procedures") {
			// Stored proc text should be in txtResults
			// Parse stored proc params if not already done so
			if (sproc_params.count(m_sproc) != 1)
				parse_sproc();

			if (sproc_params[m_sproc].size() > 0)
				mnuSProcParam->popup(pt);
			else
				mnuSProc->popup(pt);
		}
	}
	else
		sb->message("Invalid click point");
}

void DB_browser::gridMousePressed(int row, int col, int button, const QPoint& mousePos)
{
	if (button == RightButton) {
		clickrow = row;
		clickcol = col;
		mnuGrid->popup(mapToGlobal(mousePos));	// this doesn't always work
	}
}

void DB_browser::gridMouseReleased(int row, int col, int button, const QPoint& mousePos)
{
	if (button != LeftButton) return;

	QTableSelection ts = tblResults->selection(0);
	/*QString s;
	s.sprintf("Released: num = %d, (%d, %d, %d, %d)", tblResults->numSelections(), ts.leftCol(), ts.topRow(), ts.rightCol(), ts.bottomRow());
	lblMessage->setText(s); */

	QClipboard* cb = QApplication::clipboard();
	QString text;
	uint size = ts.rightCol();

	for (uint row=ts.topRow(); row <= ts.bottomRow(); row++)
		for (uint col=ts.leftCol(); col <= size; col++)
			text += tblResults->text(row, col) + (col < size ? '\t' : '\n');

	if (ts.topRow() == ts.bottomRow()) text[text.length()-1] = '\0';	// don't want \n for single row

	cb->setText(text);
}

void DB_browser::init_sql()
{
	char* p = getenv("SYBASE");
	if (!p) {
  		QMessageBox::critical(this, "Database access error", "SYBASE environment variable not set");
  		//closeAllWindows();
  		exit(1);
	}
	QString s = p;
	s += "/interfaces";
	QFile f(s);
	if (!f.open(IO_ReadOnly)) {
  		QMessageBox::critical(this, "Database access error", QString("SYBASE environment variable set, but could not read interfaces file (") + s + ")");
  		//closeAllWindows();
  		exit(1);
	}

    // file opened successfully
    QTextStream t(&f);        // use a text stream
    while (!t.eof()) {        // until end of file...
        s = t.readLine();       // line of text excluding '\n'
        if (s.isEmpty() || s[0] == ' ' || s[0] == '\t' || s[0] == '#') continue;	// ignore whitespace and comments
        if (s.find("BACK") != -1 || s.find("BCK") != -1)  continue;	// ignore Sybase backup servers (not fool-proof)

        lstServer->insertItem(s);
    }
    f.close();

}

void DB_browser::changeMode(const QString& selected)
{
	openDatabase(m_database);

	if (cmbMode->currentText() == "Tables" || cmbMode->currentText() == "System tables") {
		btnView->show();
		cmbMode->setGeometry(60,10,120,25);
	}
	else {
		btnView->hide();
		cmbMode->setGeometry(60,10,170,25);
	}
}

void DB_browser::changeViewMode()
{
	if (btnView->text() == "DDL")
		btnView->setText("Data");
	else
		btnView->setText("DDL");

	showText(m_table.c_str());	// need to check mode
}

string DB_browser::get_settings_filename()
{
	char* p = getenv("HOME");
	if (!p) return "";

	string s=p;
	s += "/.sybesquerc";
	return s;
}

/* Format of settings file is:
	[login]
	server[.database]\tusername\tpassword
As a special case, if server is *, username/password will be the default. */
void DB_browser::load_settings()
{
	string fname = get_settings_filename();
	if (fname == "") return;

	FILE* f;
	if ((f = fopen(fname.c_str(), "r")) == NULL)
		return;	// not necessarily an error

	char s[1000];
	fgets(s, 1000, f);
	if (strncmp(s, "[login]", 7) != 0) {
		QString s = "Corrupt settings file: ";
		s += fname.c_str();
		s += ". File should begin \"[login]\"";

  		QMessageBox::warning(this, "Load settings failed", s);
		return;
	}
	
	int lineno=1;
	while (fgets(s, 1000, f) != NULL) {
		lineno++;
		size_t len = strlen(s);
		if (len > 990) {	// quick and dirty check
			QString s;
			s.sprintf("Corrupt settings file: %s. Line %d too long", fname.c_str(), lineno);

			QMessageBox::warning(this, "Load settings failed", s);
			return;
		}

		if (s[0] == '#' || len < 2) continue;	// comment lines begin with #

		char* pos = strchr(s, '\t');
		if (pos == NULL || len < 5 || pos-s > len-2 || strchr(pos+1, '\t') == NULL || *(pos+1) == '\t') {
			QString s;
			s.sprintf("Corrupt settings file: %s. Line %d incorrectly formatted", fname.c_str(), lineno);

			QMessageBox::warning(this, "Load settings failed", s);
			return;
		}

		*pos = '\0';
		s[len-1] = '\0';	// remove \n

		logins[s] = pos+1;
	}
}

// Return false if failed to save settings
// Rather horrible mix of C and C++ stuff, partly to make sure file is opened mode 600.
bool DB_browser::save_settings()
{
	string fname = get_settings_filename();
	if (fname == "") return false;

	int fd = creat(fname.c_str(), 0600);	// make sure plain-text logins can't be seen by anybody else
	if (fd == -1) {
		QString s = "An error occurred trying to create ";
		s += fname.c_str();
		s += " to save settings in.\nError was: ";
		s += strerror(errno);

  		QMessageBox::warning(this, "Save settings failed", s);
		return false;
	}

	FILE* f = fdopen(fd, "w");
	fprintf(f, "[login]\n");
	for (map<string, string>::const_iterator mi = logins.begin(); mi != logins.end(); ++mi)
		fprintf(f, "%s\t%s\n", mi->first.c_str(), mi->second.c_str());
	
	fclose(f);

	return true;
}

// prompt user for username/password for server/database
bool DB_browser::get_new_login(const QString& server, const QString& database, string& username, string& password)
{
	auto_ptr<LoginDlg> dlg(new LoginDlg(this, server, database, username, password, logins));
	if (dlg->exec() != QDialog::Accepted)
		return false;

	save_settings();
	return true;
}

// Return username, password for given Sybase server/database. If no login stored already, will prompt user for one.
bool DB_browser::get_login(const QString& server, const QString& database, string& username, string& password)
{
	map<string, string>::const_iterator mi;

	string s = (const char*)server;
	s += '.';
	s += (const char*)database;
	mi = logins.find(s);	// look for server.database
	if (mi != logins.end()) {
		size_t pos = mi->second.find('\t');
		username = mi->second.substr(0, pos);
		password = mi->second.substr(pos+1);
		return true;
	}

	mi = logins.find((const char*)server);
	if (mi != logins.end()) {	// look for server
		size_t pos = mi->second.find('\t');
		username = mi->second.substr(0, pos);
		password = mi->second.substr(pos+1);
		return true;
	}

	mi = logins.find("*");
	if (mi != logins.end()) {	// look for * (default for all servers)
		size_t pos = mi->second.find('\t');
		username = mi->second.substr(0, pos);
		password = mi->second.substr(pos+1);
		return true;
	}

	return get_new_login(server, database, username, password);
}

void DB_browser::openServer(const QString& server)
{
	try {

		string username, password;
		QString database="";
		if (!get_login(server, database, username, password)) return;
		// cout << "Connect " << server << ' ' << username << ' ' << password << endl;	// qq
		sqlint.connect((const char*)server, username, password);

		lstDatabase->clear();
		lstTable->clear();
		DTable* dt = sqlint.runSQL("select name from master..sysdatabases order by name");
		for (int i=0; i < dt->rows(); i++) {
			lstDatabase->insertItem(dt->getString(0, i).c_str());
		}
		delete dt;
		m_server = server;
	}
	catch (string s) {
  		QMessageBox::critical(this, "Open server - exception thrown", s.c_str());
	}
}

void DB_browser::openDatabase(const QString& database)
{
	if (!database.length()) return;


	try {
		string username, password;
		if (!get_login(m_server, database, username, password)) return;
		// cout << "Connect " << m_server << '.' << database << ' ' << username << ' ' << password << endl;	// qq
		sqlint.connect((const char*)m_server, username, password);

		lstTable->clear();

		sqlint.useDB((const char*)database);
		DTable* dt;
		if (cmbMode->currentText() == "Tables")
			dt = sqlint.runSQL("select name from sysobjects where type = 'U' order by name");
		else if (cmbMode->currentText() == "System tables")
			dt = sqlint.runSQL("select name from sysobjects where type = 'S' order by name");
		else if (cmbMode->currentText() == "Stored procedures")
			dt = sqlint.runSQL("select name from sysobjects where type = 'P' order by name");
		else if (cmbMode->currentText() == "Triggers")
			dt = sqlint.runSQL("select name from sysobjects where type = 'TR' order by name");
		else if (cmbMode->currentText() == "Views")
			dt = sqlint.runSQL("select name from sysobjects where type = 'V' order by name");
		else
			; // program inconsistency
		for (int i=0; i < dt->rows(); i++) {
			lstTable->insertItem(dt->getString(0, i).c_str());
		}
		delete dt;
		m_database = database;
	}
	catch (string s) {
  		QMessageBox::critical(this, "Open database - exception thrown", s.c_str());
	}
}

void DB_browser::showText(const QString& selected)
{
	if (!selected.length()) return;

	try {

	DTable* dt;
	QString s;
	if (cmbMode->currentText() == "Tables" || cmbMode->currentText() == "System tables") {
		m_table = (const char*)selected;
		s.sprintf("%s.%s - table %s", (const char*)m_server, (const char*)m_database, m_table.c_str());
		setCaption(s);

		// Note: button text indicates what will happen when button clicked,
		// so it is the opposite of what is displayed in this function
		if (btnView->text() == "Data") {	// show CREATE TABLE statement
			string sql = "select Column_name = isnull(c.name, 'NULL'), Type = t.name, Length = c.length, Prec = c.prec,"
				" Scale = c.scale, Nulls = convert(bit, (c.status & 8)), Default_name = object_name(c.cdefault),"
				" Rule_name = object_name(c.domain), rtype = t.type, utype = t.usertype,Ident = convert(bit,(c.status & 0x80))"
				" from syscolumns c, systypes t"
				" where c.id = object_id(\"";
			sql += (const char*)selected;
			sql += "\") and c.usertype *= t.usertype";

			//cerr << sql << endl;
			dt = sqlint.runSQL(sql);

			lblMode->setText("");

			s.sprintf("%d column%s", dt->rows(), dt->rows() == 1 ? "" : "s");
			lblColumns->setText(s);

			string str = dtable_to_DDL((const char*)selected, dt);
			populate_results(str.c_str());
			delete dt;

			sql = string("select count(*) from ") + (const char*)selected;
			//cerr << sql << endl;
			dt = sqlint.runSQL(sql);
			s.sprintf("%d row%s", dt->getInt(0, 0), dt->getInt(0, 0) == 1 ? "" : "s");
			lblRows->setText(s);
			delete dt;
		}
		else {	// show contents of table
			string sql = "select * from ";
			sql += (const char*)selected;
			//cerr << sql << endl;
			dt = sqlint.runSQL(sql);

			populate_qtable(dt);
			delete dt;
		}
	}
	else {	// stored procedures and triggers
		m_sproc = (const char*)selected;
		if (cmbMode->currentText() == "Stored procedures")
			 s.sprintf("%s.%s - stored procedure %s", (const char*)m_server, (const char*)m_database, m_sproc.c_str());
		else
			 s.sprintf("%s.%s - trigger %s", (const char*)m_server, (const char*)m_database, m_sproc.c_str());
		setCaption(s);

		lblMode->setText("");
		lblColumns->setText("");
		lblRows->setText("");
		populate_results(SQL_object_to_text((const char*)selected).c_str());
	}

	}
	catch (string s) {
  		QMessageBox::critical(this, "Select in database - exception thrown", s.c_str());
	}
}

void DB_browser::populate_qtable(const DTable* dt)
{
	int col, row;
	QString s;

	lblMode->setText("");
	s.sprintf("%d row%s", dt->rows(), dt->rows() == 1 ? "" : "s");
	lblRows->setText(s);
	s.sprintf("%d column%s", dt->columns(), dt->columns() == 1 ? "" : "s");
	lblColumns->setText(s);

	txtResults->hide();

        //tblResults->setAutoUpdate( FALSE );	// ? qq

	tblResults->setSorting(true);
	//tblResults->clear();
	tblResults->setNumCols(dt->columns());
	tblResults->setNumRows(dt->rows());

	QHeader* qh = tblResults->horizontalHeader();
	for (col=0; col < dt->columns(); col++)
		qh->setLabel(col, dt->colName(col).c_str());

	for (int row=0; row < dt->rows(); row++)
		for (int col=0; col < dt->columns(); col++)
			tblResults->setText(row, col, dt->getString(col, row).c_str());

	tblResults->show();

        // tblResults->setAutoUpdate( TRUE );	// ? qq
	// tblResults->repaint();
}

void DB_browser::populate_results(const QString& s)
{
	tblResults->hide();
	txtResults->clear();
	txtResults->setText(s);
	txtResults->show();
}

/*"select Column_name = isnull(c.name, 'NULL'), Type = t.name, Length = c.length, Prec = c.prec,"
	" Scale = c.scale, Nulls = convert(bit, (c.status & 8)), Default_name = object_name(c.cdefault),"
	" Rule_name = object_name(c.domain), rtype = t.type, utype = t.usertype,Ident = convert(bit,(c.status & 0x80))"
 " from syscolumns c, systypes t" */
string DB_browser::dtable_to_DDL(const string& tablename, const DTable* dt)
{
	ostringstream ddl;
	ddl << "CREATE TABLE " << tablename << " (\n";

	for (int row=0; row < dt->rows(); row++) {
		string type = dt->getString(1, row);
// 		ddl << '\t' << setw(15) << left << dt->getString(0, row) << '\t' << type;
		char ss[200];
		sprintf(ss, "\t%-15s\t%s", dt->getString(0, row).c_str(), type.c_str());
		ddl << ss;
		//ddl << '\t' << std::setw(15) << dt->getString(0, row) << '\t' << type;	// qq: left removed
		if (type == "char" || type == "varchar")
			ddl << '(' << dt->getInt(2, row) << ')';
		else if (type == "numeric")
			ddl << '(' << dt->getInt(3, row) << ',' << dt->getInt(4, row) << ')';

		if (dt->getString(6, row) != "")
			ddl << '\t' << (type == "int" ? "\t" : "") << getDefaultText(dt->getString(6, row));

//		ddl << (dt->getInt(5, row) ? " NULL" : " NOT NULL");
		if (dt->getInt(5, row)) ddl << " NULL";	// otherwise NOT NULL is assumed (makes DDL clearer)

		if (row != dt->rows()-1) ddl << ',';
		ddl << '\n';
	}

	ddl << ")\ngo\n";

	return ddl.str();
}

string DB_browser::SQL_object_to_text(const string& SQL_obj)
{
	string sql = "select text from syscomments where id = object_id(\"" + SQL_obj + "\") order by number, colid2, colid";
	DTable* dt = sqlint.runSQL(sql);

	string text;
	for (int i=0; i < dt->rows(); i++)
		text += dt->getString(0, i).c_str();

	delete dt;

	return text;
}

string DB_browser::getDefaultText(const string& defaultname)
{
	string dftext = SQL_object_to_text(defaultname);

	while (dftext.length() > 0 && isspace(dftext[dftext.length()-1]))
		dftext.erase(dftext.end()-1);	// strip trailing whitespace (problem with rtrim)
	return dftext;
}

void DB_browser::LoginDB()
{
	string username, password;
	get_new_login(m_server, m_database, username, password);
	openDatabase(m_database);	// now login with new details
}

void DB_browser::DumpDDLDir()
{
	// get a directory name
	QString f = QFileDialog::getSaveFileName(QString::null, "*", this);
	if (f.isEmpty())	// the user cancelled the dialog
		return;

	// create directory
	mkdir((const char*)f, 0777);

	// get list of tables
	ofstream of;
	DTable* lst = sqlint.runSQL("select name from sysobjects where type = 'U' order by name");
	for (int i=0; i < lst->rows(); i++) {
		// open file
		of.open((const char*)(f + "/" + lst->getString(0, i).c_str()));

		lblMode->setText(lst->getString(0, i).c_str());

		// get DDL
		string sql = "select Column_name = isnull(c.name, 'NULL'), Type = t.name, Length = c.length, Prec = c.prec,"
			" Scale = c.scale, Nulls = convert(bit, (c.status & 8)), Default_name = object_name(c.cdefault),"
			" Rule_name = object_name(c.domain), rtype = t.type, utype = t.usertype,Ident = convert(bit,(c.status & 0x80))"
			" from syscolumns c, systypes t"
			" where c.id = object_id(\"";
		sql += lst->getString(0, i);
		sql += "\") and c.usertype *= t.usertype";

		DTable* dt = sqlint.runSQL(sql);
		of << dtable_to_DDL(lst->getString(0, i), dt);
		delete dt;

		// close file
		of.close();
	}
	delete lst;
}

void DB_browser::DumpDDLFile()
{
	// get a file name
	QString f = QFileDialog::getSaveFileName(QString::null, "*", this);
	if (f.isEmpty())	// the user cancelled the dialog
		return;

	ofstream of((const char*)f);

	// get list of tables
	DTable* lst = sqlint.runSQL("select name from sysobjects where type = 'U' order by name");
	for (int i=0; i < lst->rows(); i++) {
		lblMode->setText(lst->getString(0, i).c_str());
		lblMode->repaint();

		// get DDL
		string sql = "select Column_name = isnull(c.name, 'NULL'), Type = t.name, Length = c.length, Prec = c.prec,"
			" Scale = c.scale, Nulls = convert(bit, (c.status & 8)), Default_name = object_name(c.cdefault),"
			" Rule_name = object_name(c.domain), rtype = t.type, utype = t.usertype,Ident = convert(bit,(c.status & 0x80))"
			" from syscolumns c, systypes t"
			" where c.id = object_id(\"";
		sql += lst->getString(0, i);
		sql += "\") and c.usertype *= t.usertype";

		DTable* dt = sqlint.runSQL(sql);
		of << dtable_to_DDL(lst->getString(0, i), dt) << "\n";
		delete dt;
	}

	// close file
	of.close();

	delete lst;
}

void DB_browser::DumpSPDir()
{
	// get a directory name
	QString f = QFileDialog::getSaveFileName(QString::null, "*", this);
	if (f.isEmpty())	// the user cancelled the dialog
		return;

	// create directory
	mkdir((const char*)f, 0777);

	// get list of tables
	ofstream of;
	DTable* lst = sqlint.runSQL("select name from sysobjects where type = 'P' order by name");
	for (int i=0; i < lst->rows(); i++) {
		// open file
		of.open((const char*)(f + "/" + lst->getString(0, i).c_str()));

		lblMode->setText(lst->getString(0, i).c_str());

		// get text
		of << SQL_object_to_text(lst->getString(0, i)) << "go\n";

		// close file
		of.close();
	}
	delete lst;
}

void DB_browser::DumpSPFile()
{
	// get a file name
	QString f = QFileDialog::getSaveFileName(QString::null, "*", this);
	if (f.isEmpty())	// the user cancelled the dialog
		return;

	ofstream of((const char*)f);

	// get list of tables
	DTable* lst = sqlint.runSQL("select name from sysobjects where type = 'P' order by name");
	for (int i=0; i < lst->rows(); i++) {
		lblMode->setText(lst->getString(0, i).c_str());
		lblMode->repaint();

		// get text
		of << SQL_object_to_text(lst->getString(0, i)) << "go\n\n";
	}

	// close file
	of.close();

	delete lst;
}

void DB_browser::DumpDataDir()
{
	// get a directory name
	//QString f = QFileDialog::getSaveFileName(QString::null, "*", this);
	QString f = QFileDialog::getExistingDirectory();
	if (f.isEmpty())	// the user cancelled the dialog
		return;

	// create directory
	mkdir((const char*)f, 0777);

	// get list of tables
	ofstream of;
	DTable* lst = sqlint.runSQL("select name from sysobjects where type = 'U' order by name");
	for (int i=0; i < lst->rows(); i++) {
		lblMode->setText(lst->getString(0, i).c_str());
		lblMode->repaint();

		string login = (m_server == "DSFOXLDT01" || m_server == "DSFOXLDL01") ? "blotteradm" : "foxadm";

		string cmd = string("bcp ") + (const char*)m_database + ".." + lst->getString(0, i) + " out "
		    + (const char*)f + "/" + lst->getString(0, i) + ".dat -c -S " + (const char*)m_server
		    + " -U " + login + " -P " + login;
		cerr << cmd;

		system(cmd.c_str());
	}

	delete lst;
}

void DB_browser::CopyList()
{
	QClipboard* cb = QApplication::clipboard();
	QString text;
	uint size = lstTable->count();

	for (uint i=0; i<size; i++)
		text += lstTable->text(i) + "\n";

	cb->setText(text);
}

void DB_browser::parse_table()
{
	string sql = "select c.name, t.name 'type' from syscolumns c, systypes t where c.id = object_id('";
	sql += m_table;
	sql += "') and c.usertype *= t.usertype";

	DTable* dt = sqlint.runSQL(sql);

	TableQuery tquery(m_table);
	for (int i=0; i < dt->rows(); i++) {
		tquery.col_names.push_back(dt->getString(0, i));
		tquery.col_types.push_back(dt->getString(1, i));

		if (i>0) tquery.select += ", ";
		tquery.select += tquery.col_names[i];
	}

	table_queries[m_table] = tquery;
}

void DB_browser::ExecQuery()
{
	// Stored proc text should be in txtResults
	// Parse stored proc params if not already done so
	if (table_queries.count(m_table) != 1)
		parse_table();

	try {
        	string sql;
		// display dialog box with list of params
		auto_ptr<SqlQueryDlg> dlg(new SqlQueryDlg(this, table_queries[m_table]));
		if (dlg->exec() != QDialog::Accepted)
			return;

		sql = dlg->getSQL();
		cerr << sql << endl;

		// execute query
		DTable* dt = sqlint.runSQL(sql);

		// display results, rows affected and/or return code
		populate_qtable(dt);
		delete dt;
	}
	catch (string s) {
  		QMessageBox::critical(this, "Select in database - exception thrown", s.c_str());
	}
}

void DB_browser::ExecUpdate()
{
	// Stored proc text should be in txtResults
	// Parse stored proc params if not already done so
	if (table_queries.count(m_table) != 1)
		parse_table();

	try {
        	string sql;
		// display dialog box with list of params
		auto_ptr<SqlUpdateDlg> dlg(new SqlUpdateDlg(this, table_queries[m_table]));
		if (dlg->exec() != QDialog::Accepted)
			return;

		sql = dlg->getSQL();
		cerr << sql << endl;

		// doesn't make sense to get a dtable - use cmdSQL - but want return status (qq)
		// execute update
		sqlint.cmdSQL(sql);

		// display results, rows affected and/or return code
		QString s;
		s.sprintf("%d row%s affected", sqlint.getRowsAffected(), sqlint.getRowsAffected() == 1 ? "" : "s");
		lblMode->setText(s);
		RefreshResults();
	}
	catch (string s) {
  		QMessageBox::critical(this, "Update in database - exception thrown", s.c_str());
	}
}

void DB_browser::ExecDelete()
{
	// Stored proc text should be in txtResults
	// Parse stored proc params if not already done so
	if (table_queries.count(m_table) != 1)
		parse_table();

	try {
        	string sql;
		// display dialog box with list of params
		auto_ptr<SqlDeleteDlg> dlg(new SqlDeleteDlg(this, table_queries[m_table]));
		if (dlg->exec() != QDialog::Accepted)
			return;

		sql = dlg->getSQL();
		cerr << sql << endl;

		// doesn't make sense to get a dtable - use cmdSQL - but want return status (qq)
		// execute update
		sqlint.cmdSQL(sql);

		// display results, rows affected and/or return code
		QString s;
		s.sprintf("%d row%s affected", sqlint.getRowsAffected(), sqlint.getRowsAffected() == 1 ? "" : "s");
		lblMode->setText(s);
		RefreshResults();
	}
	catch (string s) {
  		QMessageBox::critical(this, "Delete in database - exception thrown", s.c_str());
	}
}

void DB_browser::ExecTruncate()
{
	 if (QMessageBox::warning(this, QString("Truncate ") + m_table.c_str(),
		QString("This will delete all rows in ") + m_table.c_str() + ",\n"
		"NOT place entries in the Sybase log\n"
		"and bypass any delete triggers on this table.\n",
		"Cancel", "Continue", 0, 1, 0) == 1)
	{
		string sql = "TRUNCATE TABLE ";
		sql += m_table;

		try {
			sqlint.cmdSQL(sql);	// would like a return status
			RefreshResults();
		}
		catch (string s) {
	  		QMessageBox::critical(this, "Truncate in database - exception thrown", s.c_str());
		}
	}
}

void DB_browser::ExecDropTable()
{
	 if (QMessageBox::warning(this, QString("Drop table ") + m_table.c_str(),
		QString("This will drop the table ") + m_table.c_str() + ".\n"
		"This action is not reversible.\n",
		"Cancel", "Continue", 0, 1, 0) == 1)
	{
		string sql = "DROP TABLE ";
		sql += m_table;

		try {
			sqlint.cmdSQL(sql);	// would like a return status
			openDatabase(m_database);	// need to refresh table list

			//RefreshResults();
		}
		catch (string s) {
	  		QMessageBox::critical(this, "drop table in database - exception thrown", s.c_str());
		}
	}
}

void DB_browser::bcpIn()
{
}

void DB_browser::bcpOut()
{
}

void DB_browser::ExecSql()
{
	// Stored proc text should be in txtResults
	// Parse stored proc params if not already done so
	if (table_queries.count(m_table) != 1)
		parse_table();

	try {
		auto_ptr<SqlSqlDlg> dlg(new SqlSqlDlg(this, m_table, query_history));
		if (dlg->exec() != QDialog::Accepted)
			return;

		string sql = dlg->getSQL();
		if (query_history.size() == 0 || query_history[query_history.size()-1] != sql)
			query_history.push_back(sql);
		cerr << sql << endl;

		DTable* dt = sqlint.runSQL(sql);

		QString s;
		s.sprintf("%d row%s affected", sqlint.getRowsAffected(), sqlint.getRowsAffected() == 1 ? "" : "s");
		lblMode->setText(s);
		populate_qtable(dt);
		delete dt;
	}
	catch (string s) {
  		QMessageBox::critical(this, "Execute SQL - exception thrown", s.c_str());
	}
}

void DB_browser::ExecSProc()
{
}

void DB_browser::parse_sproc()
{
	string sql = "select c.name 'param', t.name 'type' from syscolumns c, sysobjects o, systypes t "
		     "where o.id = c.id and c.usertype = t.usertype and o.name = '";
	sql += m_sproc;
	sql += "' order by c.colid";

	DTable* dt = sqlint.runSQL(sql);

	vector<SProcParam> params;
	for (int i=0; i < dt->rows(); i++) {
		SProcParam sp(dt->getString(0, i), dt->getString(1, i));
		params.push_back(sp);
	}

	sproc_params[m_sproc] = params;
}


void DB_browser::ExecSProcParam()
{
	try {
        	string sql;
		if (sproc_params[m_sproc].size() > 0) {
			// display dialog box with list of params
			auto_ptr<SProcDlg> dlg(new SProcDlg(this, m_sproc, sproc_params[m_sproc]));
			if (dlg->exec() != QDialog::Accepted)
				return;

			sql = dlg->getSQL();
		}
		else {
			sql = "exec " + m_sproc;
		}
		cerr << sql << endl;

		// execute stored proc
		DTable* dt = sqlint.runSQL(sql);

		// display results, rows affected and/or return code
		populate_qtable(dt);
		delete dt;

		QString s;
		s.sprintf("Return status: %d", sqlint.getReturnStatus());
		//sb->message(s);
		//lblMessage->setText(s);
		lblMode->setText(s);
	}
	catch (string s) {
  		QMessageBox::critical(this, "Execute stored procedure in database - exception thrown", s.c_str());
	}
}

void DB_browser::CopyCell()
{
	QClipboard* cb = QApplication::clipboard();
	cb->setText(tblResults->text(clickrow, clickcol));
}

void DB_browser::CopyRow()
{
	QClipboard* cb = QApplication::clipboard();
	QString text;

	for (uint i=0; i < tblResults->numCols(); i++)
		text += tblResults->text(clickrow, i) + '\t';
	text[text.length()-1] = '\0';	// strip final tab

	cb->setText(text);
}

void DB_browser::CopyCol()
{
	QClipboard* cb = QApplication::clipboard();
	QString text;

	for (uint i=0; i < tblResults->numRows(); i++)
		text += tblResults->text(i, clickcol) + '\n';

	cb->setText(text);
}

void DB_browser::CopyTable()
{
	QClipboard* cb = QApplication::clipboard();
	QString text;
	uint size = tblResults->numCols();

	for (uint row=0; row < tblResults->numRows(); row++)
		for (uint col=0; col < size; col++)
			text += tblResults->text(row, col) + (col < size-1 ? '\t' : '\n');

	cb->setText(text);
}

void DB_browser::CopyColHdr()
{
	QClipboard* cb = QApplication::clipboard();
	QString text;
	QHeader* qh = tblResults->horizontalHeader();

	for (uint i=0; i < tblResults->numCols(); i++)
		text += qh->label(i) + '\t';
	text[text.length()-1] = '\0';	// strip final tab

	cb->setText(text);
}

void DB_browser::SizeCol()
{
	tblResults->adjustColumn(clickcol);
}

void DB_browser::SizeAllCol()
{
	for (int i=0; i < tblResults->numCols(); i++)
		tblResults->adjustColumn(i);
}

void DB_browser::CountCol()
{
	map<QString, int> counts;
	for (int i=0; i < tblResults->numRows(); i++)
		counts[tblResults->text(i, clickcol)]++;

	QString hdr = tblResults->horizontalHeader()->label(clickcol);

	int row;
	QString s;

	s.sprintf("%d row%s", counts.size(), counts.size() == 1 ? "" : "s");
	lblRows->setText(s);
	lblColumns->setText("2 columns");
	lblMode->setText("");

	txtResults->hide();

	tblResults->setSorting(true);
	tblResults->setNumCols(2);
	tblResults->setNumRows(counts.size());

	QHeader* qh = tblResults->horizontalHeader();
	qh->setLabel(0, hdr);
	qh->setLabel(1, "count");

	map<QString, int>::const_iterator i;
	for (row=0, i = counts.begin(); i != counts.end(); row++, i++) {
		tblResults->setText(row, 0, i->first);
		s.sprintf("%d", i->second);
		tblResults->setText(row, 1, s);
	}

	tblResults->show();
}
