/***************************************************************************
                          db_browser.h  -  description
                             -------------------
    begin                : Tue Jan 23 2001 by Tim Colgate
    copyright            : (C) 2001 Tim Colgate
 
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *

 ***************************************************************************/


#ifndef DB_BROWSER_H
#define DB_BROWSER_H

#include <map>
#include <vector>

//Generated area. DO NOT EDIT!!!(begin)
#include <qwidget.h>
#include <qlistbox.h>
#include <qmultilinedit.h>
#include <qpushbutton.h>
#include <qcombobox.h>
//Generated area. DO NOT EDIT!!!(end)

#include <qdialog.h>
#include <qtable.h>
#include <qpopupmenu.h>
#include <qstatusbar.h>
#include <qlabel.h>

#include "sybint.h"
#include "dtable.h"

#include "login_dlg.h"
#include "sproc_dlg.h"
#include "sqlquery_dlg.h"

class DBTable : public QTable {
	Q_OBJECT
	
public:
	DBTable(QWidget* parent = 0, const char* name = 0) : QTable(parent, name), lastSortCol(-1) { setSelectionMode(/*SelectionMode::*/Single); }
	void clear() { resizeData(0); }
	
signals:
	void released(int row, int col, int button, const QPoint& mousePos);
	
protected:
	void columnClicked(int col) { if (col == lastSortCol) asc = !asc; else { lastSortCol = col; asc = TRUE; } sortColumn(col, asc, true); }
        void contentsMouseReleaseEvent(QMouseEvent* e) { emit released(currentRow(), currentColumn(), e->button(), e->pos()); QTable::contentsMouseReleaseEvent(e); }

private:
	int lastSortCol;
	bool asc;
};

class DB_browser : public QDialog  {
	Q_OBJECT
public: 
	DB_browser(QWidget *parent=0, const char *name=0);
	~DB_browser();
	
	void resizeEvent(QResizeEvent*);

protected: 
	void initDialog();
	//Generated area. DO NOT EDIT!!!(begin)
	QListBox *lstServer;
	QListBox *lstDatabase;
	QListBox *lstTable;
	QMultiLineEdit *txtResults;
	QPushButton *btnClose;
	QComboBox *cmbMode;
	QPushButton *btnView;
	//Generated area. DO NOT EDIT!!!(end)
	DBTable* tblResults;	// alternative to MultiLineEdit
	
	QStatusBar* sb;
	QLabel* lblMode;	// these widgets are on the status bar
	QLabel* lblColumns;
	QLabel* lblRows;
	QLabel* lblMessage;

private:
	SybInt	sqlint;
	
	QString	m_server;
	QString	m_database;
	string m_table;
	string m_sproc;
	int clickrow, clickcol;	// when Right button clicked in grid
	
	map<string, vector<SProcParam> >	sproc_params;
	map<string, TableQuery>			table_queries;
	vector<string>				query_history;
	map<string, string>		logins;		// server[.database] -> username\tpassword
	
	QPopupMenu* mnuDB;
	QPopupMenu* mnuTable;
	QPopupMenu* mnuSProc;
	QPopupMenu* mnuSProcParam;
	QPopupMenu* mnuGrid;
	QPopupMenu* mnuResults;	// used ?? (qq)
	//void mousePressEvent(QMouseEvent* e) { if (e->button() == RightButton) mnuTable->popup(mapToGlobal(e->pos())); }
	
	void init_sql();
	void populate_qtable(const DTable* dt);
	void populate_results(const QString& s);
	string dtable_to_DDL(const string& tablename, const DTable* dt);	// make CREATE TABLE statement from column defs
	string SQL_object_to_text(const string& SQL_obj);	// read syscomments to get SP or trigger text
	string getDefaultText(const string& defaultname);	// do sp_helptext to get contents of default on DB column
	void parse_table();
	void parse_sproc();

	string get_settings_filename();
	void load_settings();
	bool save_settings();	// return false if unable to save settings
	bool get_new_login(const QString& server, const QString& database, string& username, string& password);
	bool get_login(const QString& server, const QString& database, string& username, string& password);
	
public slots:
	void changeMode(const QString& selected);	// switch btwn display table def, table contents
	void changeViewMode();	// switch btwn display table def, table contents
	void openServer(const QString& server);	
	void openDatabase(const QString& database);	
	void showText(const QString& selected);
	void DBRightClick(QListBoxItem* item, const QPoint& pt);
	void tableRightClick(QListBoxItem* item, const QPoint& pt);
	void gridMousePressed(int row, int col, int button, const QPoint& mousePos);
	void gridMouseReleased(int row, int col, int button, const QPoint& mousePos);
	
	// popup menu functions
	void LoginDB();	// database actions
	void DumpDDLDir();
	void DumpDDLFile();
	void DumpSPDir();
	void DumpSPFile();
	void DumpDataDir();
	void CopyList();
	void RefreshResults() { showText(m_table.c_str()); }	// table actions
	void ExecQuery();
	void ExecUpdate();
	void ExecDelete();
	void ExecTruncate();
	void ExecDropTable();
	void bcpIn();
	void bcpOut();
	void ExecSql();
	void ExecSProc();	// SP actions
	void ExecSProcParam();
	void CopyCell();	// grid actions
	void CopyRow();
	void CopyCol();
	void CopyTable();
	void CopyColHdr();
	void SizeCol();
	void SizeAllCol();
	void CountCol();
};

#endif
