/***************************************************************************
                          sqlquery_dlg.cpp  -  description
                             -------------------
    begin                : Wed Feb 7 2001 by Tim Colgate
    copyright            : (C) 2001 Tim Colgate

 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *

 ***************************************************************************/

#include <iostream>
#include <string>
using namespace std;

#include <qmessagebox.h>
#include <qtooltip.h>

#include "sqlquery_dlg.h"

/************************************ QUERIES ******************************/
SqlQueryDlg::SqlQueryDlg(QWidget *parent, TableQuery& tquery_) : QDialog(parent, NULL, true), tquery(tquery_)
{
	this->resize(580, 260);
//	this->setMinimumSize(0,0);
	this->setCaption(QString("Query ") + tquery.from.c_str());

	lblSelect = new QLabel("SELECT", this);
	lblSelect->setGeometry(10, 10, 90, 25);
	lblFrom = new QLabel(QString("FROM"), this);
	lblFrom->setGeometry(10, 50, 90, 25);
	lblTable = new QLabel(tquery.from.c_str(), this);
	lblTable->setGeometry(110, 50, 200, 25);
	lblWhere = new QLabel("WHERE", this);
	lblWhere->setGeometry(10, 90, 90, 25);
	lblGroupBy = new QLabel("GROUP BY", this);
	lblGroupBy->setGeometry(10, 130, 90, 25);
	lblOrderBy = new QLabel("ORDER BY", this);
	lblOrderBy->setGeometry(10, 170, 90, 25);

	edtSelect = new QLineEdit(tquery.select.c_str(), this);
	edtSelect->setGeometry(110, 10, 460, 25);
	edtSelect->setFocus();
	edtWhere = new QLineEdit(tquery.where.c_str(), this);
	edtWhere->setGeometry(110, 90, 460, 25);
	edtGroupBy = new QLineEdit(tquery.group.c_str(), this);
	edtGroupBy->setGeometry(110, 130, 460, 25);
	edtOrderBy = new QLineEdit(tquery.order.c_str(), this);
	edtOrderBy->setGeometry(110, 170, 460, 25);

	btnClear= new QPushButton("Clear Values", this);
	btnClear->setGeometry(40, 220, 100, 30);
	//QToolTip::add(btnClear, "Clear all parameter values");

	btnCancel= new QPushButton("Cancel", this);
	btnCancel->setGeometry(240, 220, 100, 30);

	btnExec= new QPushButton("Query", this);
	btnExec->setGeometry(440, 220, 100, 30);
	btnExec->setDefault(true);

	connect(btnClear, SIGNAL(clicked()), this, SLOT(clearValues()));
	connect(btnCancel, SIGNAL(clicked()), this, SLOT(reject()));
	connect(btnExec, SIGNAL(clicked()), this, SLOT(accept()));
}

void SqlQueryDlg::resizeEvent(QResizeEvent*)
{
	edtSelect->resize(width() - 120, 25);
	edtWhere->resize(width() - 120, 25);
	edtOrderBy->resize(width() - 120, 25);

	btnClear->move(width()/15, height() - 40);
	btnCancel->move(width()/2 - 50, height() - 40);
	btnExec->move(width() - 100 - width()/15, height() - 40);
}

void SqlQueryDlg::clearValues()
{
	string select;
	for (int i=0; i < tquery.col_names.size(); i++) {
		if (i>0) select += ", ";
		select += tquery.col_names[i];
	}
	edtSelect->setText(select.c_str());

	edtWhere->clear();
	edtGroupBy->clear();
	edtOrderBy->clear();
}

string SqlQueryDlg::getSQL()
{
	// save the query
	tquery.select = (const char*)edtSelect->text();
	tquery.where = (const char*)edtWhere->text();
	tquery.group = (const char*)edtGroupBy->text();
	tquery.order = (const char*)edtOrderBy->text();

	// construct the SQL
	string sql = "SELECT " + tquery.select + " FROM " + tquery.from;
	if (tquery.where != "") sql += " WHERE " + tquery.where;
	if (tquery.group != "") sql += " GROUP BY " + tquery.group;
	if (tquery.order != "") sql += " ORDER BY " + tquery.order;

	return sql;
}

/************************************ UPDATES ******************************/
SqlUpdateDlg::SqlUpdateDlg(QWidget *parent, TableQuery& tquery_) : QDialog(parent, NULL, true), tquery(tquery_)
{
	this->resize(580, 180);
//	this->setMinimumSize(0,0);
	this->setCaption(QString("Update ") + tquery.from.c_str());

	lblUpdate = new QLabel("UPDATE", this);
	lblUpdate->setGeometry(10, 10, 90, 25);
	lblTable = new QLabel(tquery.from.c_str(), this);
	lblTable->setGeometry(110, 10, 200, 25);
	lblSet = new QLabel("SET", this);
	lblSet->setGeometry(10, 50, 90, 25);
	lblWhere = new QLabel("WHERE", this);
	lblWhere->setGeometry(10, 90, 90, 25);

	edtSet = new QLineEdit(tquery.update_set.c_str(), this);
	edtSet->setGeometry(110, 50, 460, 25);
	edtSet->setFocus();
	edtWhere = new QLineEdit(tquery.update_where.c_str(), this);
	edtWhere->setGeometry(110, 90, 460, 25);


	btnClear= new QPushButton("Clear Values", this);
	btnClear->setGeometry(40, 140, 100, 30);
	//QToolTip::add(btnClear, "Clear all parameter values");

	btnCancel= new QPushButton("Cancel", this);
	btnCancel->setGeometry(240, 140, 100, 30);

	btnExec= new QPushButton("Update", this);
	btnExec->setGeometry(440, 140, 100, 30);
	btnExec->setDefault(true);

	connect(btnClear, SIGNAL(clicked()), this, SLOT(clearValues()));
	connect(btnCancel, SIGNAL(clicked()), this, SLOT(reject()));
	connect(btnExec, SIGNAL(clicked()), this, SLOT(accept()));
}

void SqlUpdateDlg::resizeEvent(QResizeEvent*)
{
	edtSet->resize(width() - 120, 25);
	edtWhere->resize(width() - 120, 25);

	btnClear->move(width()/15, height() - 40);
	btnCancel->move(width()/2 - 50, height() - 40);
	btnExec->move(width() - 100 - width()/15, height() - 40);
}

void SqlUpdateDlg::clearValues()
{
	edtSet->clear();
	edtWhere->clear();
}

string SqlUpdateDlg::getSQL()
{
	// save the update
	tquery.update_set = (const char*)edtSet->text();
	tquery.update_where = (const char*)edtWhere->text();

	// construct the SQL
	string sql = "UPDATE " + tquery.from + " SET " + tquery.update_set;
	if (tquery.update_where != "") sql += " WHERE " + tquery.update_where;

	return sql;
}

/************************************ DELETES ******************************/
SqlDeleteDlg::SqlDeleteDlg(QWidget *parent, TableQuery& tquery_) : QDialog(parent, NULL, true), tquery(tquery_)
{
	this->resize(580, 140);
//	this->setMinimumSize(0,0);
	this->setCaption(QString("Delete ") + tquery.from.c_str());

	lblDelete = new QLabel("DELETE", this);
	lblDelete->setGeometry(10, 10, 90, 25);
	lblTable = new QLabel(tquery.from.c_str(), this);
	lblTable->setGeometry(110, 10, 200, 25);
	lblWhere = new QLabel("WHERE", this);
	lblWhere->setGeometry(10, 50, 90, 25);

	edtWhere = new QLineEdit(tquery.delete_where.c_str(), this);
	edtWhere->setGeometry(110, 50, 460, 25);
	edtWhere->setFocus();

	btnClear= new QPushButton("Clear Values", this);
	btnClear->setGeometry(40, 100, 100, 30);
	//QToolTip::add(btnClear, "Clear all parameter values");

	btnCancel= new QPushButton("Cancel", this);
	btnCancel->setGeometry(240, 100, 100, 30);

	btnExec= new QPushButton("Delete", this);
	btnExec->setGeometry(440, 100, 100, 30);
	btnExec->setDefault(true);

	connect(btnClear, SIGNAL(clicked()), this, SLOT(clearValues()));
	connect(btnCancel, SIGNAL(clicked()), this, SLOT(reject()));
	connect(btnExec, SIGNAL(clicked()), this, SLOT(accept()));
}

void SqlDeleteDlg::resizeEvent(QResizeEvent*)
{
	edtWhere->resize(width() - 120, 25);

	btnClear->move(width()/15, height()-40);
	btnCancel->move(width()/2 - 50, height()-40);
	btnExec->move(width() - 100 - width()/15, height()-40);
}

void SqlDeleteDlg::clearValues()
{
	edtWhere->clear();
}

string SqlDeleteDlg::getSQL()
{
	// save the delete
	tquery.delete_where = (const char*)edtWhere->text();

	// construct the SQL
	string sql = "DELETE " + tquery.from;
	if (tquery.delete_where != "") sql += " WHERE " + tquery.delete_where;

	return sql;
}


/************************************ SQL ******************************/
SqlSqlDlg::SqlSqlDlg(QWidget *parent, const string& table_, vector<string>& history_) : QDialog(parent, NULL, true), table(table_), history(history_)
{
	this->resize(580, 300);
//	this->setMinimumSize(0,0);
	this->setCaption("Execute SQL");

	edtSql = new QMultiLineEdit(this);
	//edtSql->setText(QString("SELECT\nFROM ") + table.c_str());
	if (history.size() > 0)
		edtSql->setText(history[history.size()-1].c_str());
	edtSql->setFocus();

	cmbHistory = new QComboBox(this);
	for (int i=history.size()-1; i >= 0; i--)	// in reverse - most recent history at top
		cmbHistory->insertItem(history[i].c_str());

	btnClear= new QPushButton("&Clear", this);
	btnClear->setAccel(ALT+'C');
	btnCancel= new QPushButton("Cancel", this);
	btnExec= new QPushButton("&Execute", this);
	btnExec->setAccel(ALT+'E');
	btnExec->setDefault(true);

	connect(btnClear, SIGNAL(clicked()), this, SLOT(clearValues()));
	connect(btnCancel, SIGNAL(clicked()), this, SLOT(reject()));
	connect(btnExec, SIGNAL(clicked()), this, SLOT(accept()));
	connect(cmbHistory, SIGNAL(activated(const QString&)), this, SLOT(changeHistory(const QString&)));
}

void SqlSqlDlg::resizeEvent(QResizeEvent*)
{
	edtSql->    setGeometry(10, 10,          width()-20, height()-95);
	cmbHistory->setGeometry(10, height()-75, width()-20, 25);

	btnClear-> move(10,             height()-40);
	btnCancel->move(width()/2 - 50, height()-40);
	btnExec->  move(width()-110,    height()-40);
}

void SqlSqlDlg::clearValues()
{
	edtSql->clear();
}

void SqlSqlDlg::changeHistory(const QString& selected)
{
	edtSql->setText(selected);
}

string SqlSqlDlg::getSQL()
{
	return (const char*)edtSql->text();
}
