/***************************************************************************
                          sqlquery_dlg.h  -  description
                             -------------------
    begin                : Wed Feb 7 2001 by Tim Colgate
    copyright            : (C) 2001 Tim Colgate

 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *

 ***************************************************************************/

#ifndef SQLQUERY_DLG_H
#define SQLQUERY_DLG_H

#include <vector>
#include <string>
using namespace std ;

#include <qwidget.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qmultilinedit.h>
#include <qdialog.h>


struct TableQuery {
	TableQuery() {}		// needed because map<> requires constructor with no args
	TableQuery(string table): from(table) {}

	// these fields represent the query. Note that "from" is fixed - it's just the table name
	string select;
	string from;
	string where;
	string group;
	string order;

	// these fields represent other operations on the table, e.g. DELETE, UPDATE
	string delete_where;
	string update_set;
	string update_where;

	// these are fixed for a table
	vector<string> col_names;
	vector<string> col_types;	// not used at present
};


class SqlQueryDlg : public QDialog  {
   Q_OBJECT
public:
	SqlQueryDlg(QWidget *parent, TableQuery& tquery);
	~SqlQueryDlg() {}

	string getSQL();
	void resizeEvent(QResizeEvent*);

private:
	QPushButton *btnClear;
	QPushButton *btnCancel;
	QPushButton *btnExec;

	TableQuery& tquery;

	QLabel* lblSelect;
	QLabel* lblFrom;
	QLabel* lblTable;
	QLabel* lblWhere;
	QLabel* lblGroupBy;
	QLabel* lblOrderBy;
	QLineEdit* edtSelect;
	QLineEdit* edtWhere;
	QLineEdit* edtGroupBy;
	QLineEdit* edtOrderBy;

public slots:
	void clearValues();
};


class SqlDeleteDlg : public QDialog  {
   Q_OBJECT
public:
	SqlDeleteDlg(QWidget *parent, TableQuery& tquery);
	~SqlDeleteDlg() {}

	string getSQL();
	void resizeEvent(QResizeEvent*);

private:
	QPushButton *btnClear;
	QPushButton *btnCancel;
	QPushButton *btnExec;

	TableQuery& tquery;

	QLabel* lblDelete;
	QLabel* lblTable;
	QLabel* lblWhere;
	QLineEdit* edtWhere;

public slots:
	void clearValues();
};


class SqlUpdateDlg : public QDialog  {
   Q_OBJECT
public:
	SqlUpdateDlg(QWidget *parent, TableQuery& tquery);
	~SqlUpdateDlg() {}

	string getSQL();
	void resizeEvent(QResizeEvent*);

private:
	QPushButton *btnClear;
	QPushButton *btnCancel;
	QPushButton *btnExec;

	TableQuery& tquery;

	QLabel* lblUpdate;
	QLabel* lblTable;
	QLabel* lblSet;
	QLabel* lblWhere;
	QLineEdit* edtSet;
	QLineEdit* edtWhere;

public slots:
	void clearValues();
};


class SqlSqlDlg : public QDialog  {
   Q_OBJECT
public:
	SqlSqlDlg(QWidget *parent, const string& table, vector<string>& history);
	~SqlSqlDlg() {}

	string getSQL();
	void resizeEvent(QResizeEvent*);

private:
	QPushButton *btnClear;
	QPushButton *btnCancel;
	QPushButton *btnExec;

	const string& table;
	vector<string>& history;

	QMultiLineEdit* edtSql;
	QComboBox* cmbHistory;

public slots:
	void changeHistory(const QString& selected);
	void clearValues();
};

#endif
